﻿;/////////////////////////////////////////////////////////////////////////////////
;An OOP 'rectangle' class
;
;nxSoftware.com
;/////////////////////////////////////////////////////////////////////////////////

;-CLASS INTERFACE.  (This is the public face of our rectangle class and allows PB to call our method functions).
  Interface RectangleObject
    Area.d()
    Destroy()
    GetLength.d()
    GetWidth.d()
    IsSquare.i()
    LengthOfDiagonal.d()
    SetLength.i(newLength)
    SetWidth.i(newWidth)
  EndInterface


;-CLASS TEMPLATE.
  Structure _RectangleClassTemplate
	  *vTable
	  length.d
	  width.d
  EndStructure


;-CREATE AN INSTANCE OF A RECTANGLE CLASS.

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NewRectangleObject(length=0, width=0)
  Protected *object._RectangleClassTemplate
  ;Attempt to allocate memory for a new class template.
    *object = AllocateMemory(SizeOf(_RectangleClassTemplate))
    If *object
      ;Make sure the *vTable field points to our virtual table.
        *object\vTable = ?VTable_RectangleClass
      ;Initialise the length and width.
        *object\length = length
        *object\width = width
    EndIf
  ;Return a pointer to our object.
    ProcedureReturn *object
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;CLASS METHODS

;/////////////////////////////////////////////////////////////////////////////////
Procedure.d RectangleClass_Area(*this._RectangleClassTemplate)
  Protected area.d
  area = *this\length * *this\width
  ProcedureReturn area
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure RectangleClass_Destroy(*this._RectangleClassTemplate)
  ;All we need do here is free the memory previously allocated for this object.
    FreeMemory(*this)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.d RectangleClass_GetLength(*this._RectangleClassTemplate)
  ProcedureReturn *this\length
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.d RectangleClass_GetWidth(*this._RectangleClassTemplate)
  ProcedureReturn *this\width
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i RectangleClass_IsSquare(*this._RectangleClassTemplate)
  If *this\length = *this\width
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.d RectangleClass_LengthOfDiagonal(*this._RectangleClassTemplate)
  Protected length.d
  length = *this\length * *this\length + *this\width * *this\width
  length = Sqr(length)
  ProcedureReturn length
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns #True if the length is successfully changed.
Procedure.i RectangleClass_SetLength(*this._RectangleClassTemplate, newLength)
  ;Only proceed if the user is attempting to set a non-negative length.
  If newLength >= 0
    *this\length = newLength
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns #True if the width is successfully changed.
Procedure.i RectangleClass_SetWidth(*this._RectangleClassTemplate, newWidth)
  ;Only proceed if the user is attempting to set a non-negative width.
  If newWidth >= 0
    *this\width = newWidth
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-VIRTUAL TABLES.

DataSection 
  VTable_RectangleClass: 
    Data.i @RectangleClass_Area()
    Data.i @RectangleClass_Destroy()
    Data.i @RectangleClass_GetLength()
    Data.i @RectangleClass_GetWidth()
    Data.i @RectangleClass_IsSquare()
    Data.i @RectangleClass_LengthOfDiagonal()
    Data.i @RectangleClass_SetLength()
    Data.i @RectangleClass_SetWidth()
EndDataSection 

; IDE Options = PureBasic 4.61 (Windows - x86)
; CursorPosition = 3
; Folding = --